package services;

import l2e.commons.gson.Gson;
import l2e.commons.gson.reflect.TypeToken;
import l2e.gameserver.ThreadPoolManager;
import l2e.gameserver.data.dao.CharacterDAO;
import l2e.gameserver.database.DatabaseFactory;
import l2e.gameserver.instancemanager.ServerVariables;
import l2e.gameserver.model.GameObjectsStorage;
import l2e.gameserver.model.actor.Player;
import l2e.gameserver.model.quest.Quest;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Type;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Paths;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.logging.*;

public class l2vote extends Quest{
    public l2vote(int questId, String name, String descr) {
        super(questId, name, descr);
        System.out.println("---------L2VOTE--------");
        System.out.println("!LOADED!");
        System.out.println("---------L2VOTE--------");
        VoteProcessor.onLoad();
        // Инициализация текущей даты

    }
    public static void main(String[] args) {
        new l2vote(-1,"L2vote system","loaded!");
    }
    public class Config {
        public static  String API_URL_TEMPLATE = "https://api.l2vote.com/api/votes/server/%s/votes/today";
        public static  String API_KEY="";                                             // Укажите ваш API-ключ
        public static  int SERVER_ID=0;                                               // ID сервера
        public static  int POLLING_DELAY_MINUTES=1;                                   // Вермя задержки опроса в минутах
        public static String[] L2VoteRewardItemId = "57:1".split(";");                // Предметы за голос, можно указывать несколько, пример 57:1;4037:1
        public static boolean L2VoteRewardItemRandom=false;                           // Если включить, выдаст рандомно только 1 награду из конфига L2VoteRewardItemId
        public static boolean ALLOW_MULTIPLE_REWARDS=false;                           // Можно голосовать на 1 ник множнство раз                        
        public static boolean ENABLE_CHECK_IP=false;                                  // Включить проверку по IP
        public static boolean ENABLE_CHECK_HWID=false;                                // Включить проверку по HWID
    }

    public static class VoteProcessor  {

        private String today;

        public VoteProcessor(String today) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            this.today = sdf.format(new Date());

        }


        public static void start() {
            try (Connection connection = DatabaseFactory.getInstance().getConnection();
                 DatabaseManager dbManager = new DatabaseManager(connection)) {

                // Создаем таблицу
                dbManager.createTableIfNotExists();

                // Получаем данные из API
                ApiClient apiClient = new ApiClient();
                // System.out.println("Получение новых голосов...");
                List<Vote> votes = apiClient.fetchVotes();

                // Обрабатываем каждый голос
                for (Vote vote : votes) {
                    if (dbManager.addVote(vote.getCharName(),vote.getVoteTime(),vote.getId())) {
                        // System.out.println("Добавлен новый голос: " + vote);

                        LogManagerVote.getInstance().info("Добавлен новый голос: " + vote);
                    }
                }


                // Выдаем награды
                dbManager.giveRewards(Config.ALLOW_MULTIPLE_REWARDS);

            } catch (Exception e) {
                e.printStackTrace();
            }
        }




        public static void onLoad() {
            // Периодическое выполнение задачи с интервалом
            ThreadPoolManager.getInstance().scheduleAtFixedRate(()->start(), 5000, Config.POLLING_DELAY_MINUTES);
        }


    }



    public static class ApiClient {
        public List<Vote> fetchVotes() throws Exception {
            String url = String.format(Config.API_URL_TEMPLATE, Config.SERVER_ID);
            HttpURLConnection conn = (HttpURLConnection) new URL(url).openConnection();
            conn.setRequestMethod("GET");
            conn.setRequestProperty("X-API-KEY", Config.API_KEY);

            if (conn.getResponseCode() != 200) {
                throw new RuntimeException("API error: HTTP " + conn.getResponseCode());
            }

            BufferedReader reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            StringBuilder response = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                response.append(line);
            }
            reader.close();

            // Логируем весь ответ для понимания структуры
            // System.out.println("API Response: " + response.toString());

            // Используем Gson для парсинга JSON в список объектов Vote
            Gson gson = new Gson();
            Type listType = new TypeToken<List<Vote>>() {}.getType();
            return gson.fromJson(response.toString(), listType);  // Преобразуем в список объектов Vote
        }
    }
    public static class DatabaseManager implements AutoCloseable {
        private static Connection connection;

        public DatabaseManager(Connection connection) {
            this.connection = connection;
        }

        // Метод для создания таблицы, если она еще не существует
        public void createTableIfNotExists() throws SQLException {
            String query = "CREATE TABLE IF NOT EXISTS votes (" +
                    "id INT NOT NULL, " + // ID из API (Не нужно PRIMARY KEY, если ID не автоинкремент)
                    "char_name VARCHAR(255) NOT NULL, " +
                    "vote_time DATETIME NOT NULL, " + // Время голосования из API
                    "reward_given BOOLEAN DEFAULT FALSE, " + // Награда выдана
                    "PRIMARY KEY(id)" + // Указание PRIMARY KEY для поля id
                    ")";
            try (Statement stmt = connection.createStatement()) {
                stmt.execute(query);
            }
        }



        // Метод для добавления нового голоса в базу данных
        // Метод для добавления нового голоса
        public boolean addVote(String charName, String voteTime, int id) throws SQLException {
            String checkQuery = "SELECT 1 FROM votes WHERE id = ?";
            try (PreparedStatement ps = connection.prepareStatement(checkQuery)) {
                ps.setInt(1, id);
                ResultSet rs = ps.executeQuery();
                if (rs.next()) {
                    return false; // Голос с таким ID уже существует
                }
            }

            // Добавляем запись в таблицу голосования
            String insertQuery = "INSERT INTO votes (id, char_name, vote_time, reward_given) VALUES (?, ?, ?, FALSE)";
            try (PreparedStatement ps = connection.prepareStatement(insertQuery)) {
                ps.setInt(1, id); // ID уникален, будет использоваться для проверки
                ps.setString(2, charName);
                ps.setString(3, voteTime);
                ps.executeUpdate();
            }
            return true;
        }

        public void giveRewards(boolean allowMultipleRewards) throws SQLException {
            String today = java.time.LocalDate.now().toString(); // Текущая дата в формате yyyy-MM-dd

            // Выборка голосов с учетом allowMultipleRewards
// Выборка голосов с учетом allowMultipleRewards
            String selectQuery = allowMultipleRewards
                    ? "SELECT char_name, id FROM votes WHERE reward_given = FALSE AND DATE(vote_time) = ?"
                    : "SELECT char_name, id FROM votes WHERE reward_given = FALSE AND DATE(vote_time) = ?";  // исправлено

            try (PreparedStatement ps = connection.prepareStatement(selectQuery)) {
                ps.setString(1, today); // Устанавливаем текущую дату
                ResultSet rs = ps.executeQuery();

                while (rs.next()) {
                    String charName = rs.getString("char_name");
                    int voteId = rs.getInt("id");

                    // Проверяем, существует ли игрок с таким ником за сегодняшний день
                    String checkPlayerQuery = "SELECT COUNT(*) FROM votes WHERE char_name = ? AND DATE(vote_time) = ?";
                    try (PreparedStatement checkPlayerPs = connection.prepareStatement(checkPlayerQuery)) {
                        checkPlayerPs.setString(1, charName);
                        checkPlayerPs.setString(2, today);
                        ResultSet checkPlayerRs = checkPlayerPs.executeQuery();

                        if (checkPlayerRs.next() && checkPlayerRs.getInt(1) == 0) {
                            // Если игрока с таким ником нет в базе данных за сегодня, добавляем его
                            String insertQuery = "INSERT INTO votes (char_name, vote_time, reward_given) VALUES (?, ?, FALSE)";
                            try (PreparedStatement insertPs = connection.prepareStatement(insertQuery)) {
                                insertPs.setString(1, charName);
                                insertPs.setString(2, today);
                                insertPs.executeUpdate();
                            }

                            // Выдаем награду, так как игрока нет в базе данных
                            if(CheckIpAndHwid(charName)) {
                                grantReward(charName);
                            }else {
                                int objId = getCharIdByName(charName);
                                if(objId > 0 && GameObjectsStorage.getPlayer(objId) != null){
                                    Player player = GameObjectsStorage.getPlayer(objId);
                                    player.sendMessage("Вы уже сегодня голосовали на этот IP или HWID, попробуйте после 00:00");
                                }
                            }
                            markRewardAsGiven(charName, today);
                        } else {
                            // Если игрок есть в базе, проверяем, была ли выдана награда
                            if (!allowMultipleRewards) {
                                String checkRewardQuery = "SELECT COUNT(*) FROM votes WHERE char_name = ? AND DATE(vote_time) = ? AND reward_given = TRUE";
                                try (PreparedStatement checkPs = connection.prepareStatement(checkRewardQuery)) {
                                    checkPs.setString(1, charName);
                                    checkPs.setString(2, today);
                                    ResultSet checkRs = checkPs.executeQuery();

                                    if (checkRs.next() && checkRs.getInt(1) == 0) {
                                        // Если награда еще не выдана, выдаем награду
                                        if(CheckIpAndHwid(charName)) {
                                            grantReward(charName);
                                        }else {
                                            int objId = getCharIdByName(charName);
                                            if(objId > 0 && GameObjectsStorage.getPlayer(objId) != null){
                                                Player player = GameObjectsStorage.getPlayer(objId);
                                                player.sendMessage("Вы уже сегодня голосовали на этот IP или HWID, попробуйте после 00:00");
                                            }
                                        }
                                        markRewardAsGiven(charName, today);
                                    }else{
                                        int objId = getCharIdByName(charName);
                                        if(objId > 0 && GameObjectsStorage.getPlayer(objId) != null){
                                            Player player = GameObjectsStorage.getPlayer(objId);
                                            player.sendMessage("Вы уже сегодня голосовали на этот ник, попробуйте после 00:00");
                                        }
                                    }
                                }
                            } else {
                                // Если allowMultipleRewards == true, проверяем, был ли уже обработан голос с таким ID
                                String checkVoteQuery = "SELECT COUNT(*) FROM votes WHERE id = ? AND reward_given = TRUE";
                                try (PreparedStatement checkVotePs = connection.prepareStatement(checkVoteQuery)) {
                                    checkVotePs.setInt(1, voteId);
                                    ResultSet checkVoteRs = checkVotePs.executeQuery();

                                    if (checkVoteRs.next() && checkVoteRs.getInt(1) == 0) {
                                        // Если награда за этот голос еще не выдана, выдаем награду
                                        if(CheckIpAndHwid(charName)) {
                                            grantReward(charName);
                                        }else {
                                            int objId = getCharIdByName(charName);
                                            if(objId > 0 && GameObjectsStorage.getPlayer(objId) != null){
                                                Player player = GameObjectsStorage.getPlayer(objId);
                                                player.sendMessage("Вы уже сегодня голосовали на этот IP или HWID, попробуйте после 00:00");
                                            }
                                        }
                                        markRewardAsGiven(charName, today);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        public boolean CheckIpAndHwid(String charName) throws SQLException {
            String ip = null;
            String hwid = null;

            String getAccountNameQuery = "SELECT ip, hwid FROM characters WHERE char_name = ?";

            if (connection == null || connection.isClosed()) {
                throw new IllegalStateException("Database connection is not initialized or closed.");
            }

            try {

                try (PreparedStatement ps1 = connection.prepareStatement(getAccountNameQuery)) {
                    ps1.setString(1, charName);
                    try (ResultSet rs1 = ps1.executeQuery()) {
                        if (rs1.next()) {
                            ip = rs1.getString("ip");
                            hwid = rs1.getString("hwid");
                        }
                    }
                }




                // Шаг 3: Проверка IP и HWID
                if (Config.ENABLE_CHECK_IP && ip != null) {
                    String key = "l2vote_ip_" + ip;
                    String storedDate = ServerVariables.getString(key,"null");
                    if (storedDate != null && !storedDate.equals("null")) {
                        LocalDate localDate = LocalDate.parse(storedDate);
                        if (localDate.equals(LocalDate.now())) {
                            return false;
                        }
                    }
                    ServerVariables.set(key, LocalDate.now().toString());
                }

                if (Config.ENABLE_CHECK_HWID && hwid != null && !hwid.equals("N/A")) {
                    String key = "l2vote_hwid_" + hwid;

                    String storedDate = ServerVariables.getString(key,"null");
                    if (storedDate != null && !storedDate.equals("null")) {
                        LocalDate localDate = LocalDate.parse(storedDate);
                        if (localDate.equals(LocalDate.now())) {
                            return false;
                        }
                    }
                    ServerVariables.set(key, LocalDate.now().toString());

                }

                return true;

            } catch (SQLException e) {
                e.printStackTrace();
                throw e; // Бросаем исключение для обработки на более высоком уровне
            } catch (Exception e) {
                e.printStackTrace();
                return false;
            }
        }





        public static int getCharIdByName(String charName) throws SQLException {
            String query = "SELECT charId FROM characters WHERE char_name = ?";

            try (PreparedStatement ps = connection.prepareStatement(query)) {
                ps.setString(1, charName); // Устанавливаем имя игрока в запрос
                ResultSet rs = ps.executeQuery();

                if (rs.next()) {
                    return rs.getInt("charId"); // Возвращаем charId
                }
            }

            // Если игрок не найден, возвращаем null
            return 0;
        }

        public static void addItemToDelayed(
                int ownerId,
                int itemId,
                int count,
                int enchantLevel,
                int paymentStatus,
                String description
        ) throws SQLException {
            // SQL-запрос для вставки данных в таблицу items_delayed
            String insertQuery = "INSERT INTO items_delayed (owner_id, item_id, count, enchant_level, payment_status, description) " +
                    "VALUES (?, ?, ?, ?, ?, ?)";

            try (PreparedStatement ps = connection.prepareStatement(insertQuery)) {
                // Устанавливаем параметры для запроса
                ps.setInt(1, ownerId);                   // ID владельца
                ps.setInt(2, itemId);                    // ID предмета
                ps.setInt(3, count);                     // Количество
                ps.setInt(4, enchantLevel);              // Уровень зачарования
                ps.setInt(5, paymentStatus);         // Статус оплаты (true/false)
                ps.setString(6, description);            // Описание

                // Выполняем запрос
                ps.executeUpdate();
            }
        }

        private static void grantReward(String charName) throws SQLException {
            // Логика выдачи бонуса игроку

            int obj_id = getCharIdByName(charName);
            if(obj_id > 0) {
                if(!Config.L2VoteRewardItemRandom) {
                    for (String string : Config.L2VoteRewardItemId) {
                        String[] str = string.split(":");
                        int id = Integer.parseInt(str[0]);
                        int count = Integer.parseInt(str[1]);
                        addItemToDelayed(obj_id,id,count,0,1,"l2vote");

                        LogManagerVote.getInstance().info("Выдача награды игроку (ID: "+id+", Name: "+charName+")" );
                    }
                }else{
                    Random random = new Random();
                    int index = random.nextInt(Config.L2VoteRewardItemId.length);
                    String[] strings = Config.L2VoteRewardItemId[index].split(":");
                    int id = Integer.parseInt(strings[0]);
                    long count = Integer.parseInt(strings[1]);
                    addItemToDelayed(obj_id,id, (int) count,0,1,"l2vote");
                    //System.out.printf("Выдача награды игроку (ID: %d, Name: %s)%n", id, charName);
                    LogManagerVote.getInstance().info("Выдача награды игроку (ID: "+id+", Name: "+charName+")" );
                }
            }else{
                // System.err.println("Ошибка при выдаче наград: Игрок с ником - "+charName +" не найден!");

                LogManagerVote.getInstance().error("Ошибка при выдаче наград: Игрок с ником - "+charName +" не найден!" );
            }
            LogManagerVote.getInstance().info("Выдан бонус игроку " + charName);

        }

        private void markRewardAsGiven(String charName, String today) throws SQLException {
            String updateQuery = "UPDATE votes SET reward_given = TRUE WHERE char_name = ? AND DATE(vote_time) = ?";
            try (PreparedStatement ps = connection.prepareStatement(updateQuery)) {
                ps.setString(1, charName);
                ps.setString(2, today); // Устанавливаем текущую дату
                ps.executeUpdate();
            }
        }

        @Override
        public void close() throws SQLException {
            if (connection != null) {
                connection.close();
            }
        }


        public void addReward(String charName){

        }


    }
    public static class LogManagerVote {

        private static LogManagerVote instance; // Экземпляр Singleton
        private Logger logger;

        // Закрытый конструктор, чтобы никто не мог создать объект напрямую
        private LogManagerVote() {
            try {
                // Получаем абсолютный путь к текущей директории
                String logFilePath = Paths.get("").toAbsolutePath().toString() + "/votes.log";

                // Создаем логгер
                logger = Logger.getLogger(LogManagerVote.class.getName());

                // Очищаем существующие обработчики (чтобы избежать вывода в консоль)
                Handler[] handlers = logger.getHandlers();
                for (Handler handler : handlers) {
                    logger.removeHandler(handler);
                }

                // Устанавливаем FileHandler для записи логов в файл
                FileHandler fileHandler = new FileHandler(logFilePath, true); // append = true для добавления в конец файла
                fileHandler.setFormatter(new SimpleFormatter());  // Устанавливаем простой формат вывода

                // Добавляем обработчик в логгер
                logger.addHandler(fileHandler);
                logger.setLevel(Level.INFO);  // Устанавливаем уровень логирования

            } catch (IOException e) {
                System.err.println("Ошибка при настройке логгера: " + e.getMessage());
            }
        }

        // Метод для получения единственного экземпляра LogManager
        public static synchronized LogManagerVote getInstance() {
            if (instance == null) {
                instance = new LogManagerVote(); // Создаем экземпляр, если он еще не был создан
            }
            return instance;
        }

        // Метод для записи информационного лога
        public void info(String message) {
            logger.info(message);
        }

        // Метод для записи ошибки
        public void error(String message) {
            logger.severe(message);
        }

        // Метод для записи предупреждения
        public void warning(String message) {
            logger.warning(message);
        }

        // Метод для записи общего лога
        public void log(Level level, String message) {
            logger.log(level, message);
        }
    }
    public static class Vote {
        private int id;
        private String charName;
        private String voteTime;

        // Геттеры и сеттеры
        public int getId() {
            return id;
        }

        public void setId(int id) {
            this.id = id;
        }

        public String getCharName() {
            return charName;
        }

        public void setCharName(String charName) {
            this.charName = charName;
        }

        public String getVoteTime() {
            return voteTime;
        }

        public void setVoteTime(String voteTime) {
            this.voteTime = voteTime;
        }

        @Override
        public String toString() {
            return "Vote{" +
                    "id=" + id +
                    ", charName='" + charName + '\'' +
                    ", voteTime='" + voteTime + '\'' +
                    '}';
        }
    }


}
